@@ -25,6 +25,8 @@ cSatipTuner::cSatipTuner(cSatipDeviceIf &deviceP, unsigned int packetLenP)
   rtcpM(*this),
   streamAddrM(""),
   streamParamM(""),
+  tnrParamM(""),
+  streamPortM(SATIP_DEFAULT_RTSP_PORT),
   currentServerM(NULL, deviceP.GetId(), 0),
   nextServerM(NULL, deviceP.GetId(), 0),
   mutexM(),
@@ -50,12 +52,16 @@ cSatipTuner::cSatipTuner(cSatipDeviceIf &deviceP, unsigned int packetLenP)
   debug1("%s (, %d) [device %d]", __PRETTY_FUNCTION__, packetLenP, deviceIdM);
 
   // Open sockets
-  int i = 100;
+  int i = SatipConfig.GetPortRangeStart() ? SatipConfig.GetPortRangeStop() - SatipConfig.GetPortRangeStart() - 1 : 100;
+  int port = SatipConfig.GetPortRangeStart();
   while (i-- > 0) {
-        if (rtpM.Open(0) && rtcpM.Open(rtpM.Port() + 1))
+        // RTP must use an even port number
+        if (rtpM.Open(port) && (rtpM.Port() % 2 == 0) && rtcpM.Open(rtpM.Port() + 1))
            break;
         rtpM.Close();
         rtcpM.Close();
+        if (SatipConfig.GetPortRangeStart())
+           port += 2;
         }
   if ((rtpM.Port() <= 0) || (rtcpM.Port() <= 0)) {
      error("Cannot open required RTP/RTCP ports [device %d]", deviceIdM);
@@ -205,7 +211,8 @@ bool cSatipTuner::Connect(void)
   debug1("%s [device %d]", __PRETTY_FUNCTION__, deviceIdM);
 
   if (!isempty(*streamAddrM)) {
-     cString connectionUri = cString::sprintf("rtsp://%s/", *streamAddrM);
+     cString connectionUri = GetBaseUrl(*streamAddrM, streamPortM);
+     tnrParamM = "";
      // Just retune
      if (streamIdM >= 0) {
         cString uri = cString::sprintf("%sstream=%d?%s", *connectionUri, streamIdM, *streamParamM);
@@ -217,10 +224,11 @@ bool cSatipTuner::Connect(void)
         }
      else if (rtspM.Options(*connectionUri)) {
         cString uri = cString::sprintf("%s?%s", *connectionUri, *streamParamM);
+        bool useTcp = SatipConfig.IsTransportModeRtpOverTcp() && nextServerM.IsValid() && nextServerM.IsQuirk(cSatipServer::eSatipQuirkRtpOverTcp);
         // Flush any old content
         //rtpM.Flush();
         //rtcpM.Flush();
-        if (rtspM.Setup(*uri, rtpM.Port(), rtcpM.Port())) {
+        if (rtspM.Setup(*uri, rtpM.Port(), rtcpM.Port(), useTcp)) {
            keepAliveM.Set(timeoutM);
            if (nextServerM.IsValid()) {
               currentServerM = nextServerM;
@@ -244,7 +252,7 @@ bool cSatipTuner::Disconnect(void)
   debug1("%s [device %d]", __PRETTY_FUNCTION__, deviceIdM);
 
   if (!isempty(*streamAddrM) && (streamIdM >= 0)) {
-     cString uri = cString::sprintf("rtsp://%s/stream=%d", *streamAddrM, streamIdM);
+     cString uri = cString::sprintf("%sstream=%d", *GetBaseUrl(*streamAddrM, streamPortM), streamIdM);
      rtspM.Teardown(*uri);
      // some devices requires a teardown for TCP connection also
      rtspM.Reset();
@@ -289,6 +297,11 @@ void cSatipTuner::ProcessVideoData(u_char *bufferP, int lengthP)
   reConnectM.Set(eConnectTimeoutMs);
 }
 
+void cSatipTuner::ProcessRtpData(u_char *bufferP, int lengthP)
+{
+  rtpM.Process(bufferP, lengthP);
+}
+
 void cSatipTuner::ProcessApplicationData(u_char *bufferP, int lengthP)
 {
   debug16("%s (%d) [device %d]", __PRETTY_FUNCTION__, lengthP, deviceIdM);
@@ -342,6 +355,11 @@ void cSatipTuner::ProcessApplicationData(u_char *bufferP, int lengthP)
   reConnectM.Set(eConnectTimeoutMs);
 }
 
+void cSatipTuner::ProcessRtcpData(u_char *bufferP, int lengthP)
+{
+  rtcpM.Process(bufferP, lengthP);
+}
+
 void cSatipTuner::SetStreamId(int streamIdP)
 {
   cMutexLock MutexLock(&mutexM);
@@ -359,6 +377,47 @@ void cSatipTuner::SetSessionTimeout(const char *sessionP, int timeoutP)
   timeoutM = (timeoutP > eMinKeepAliveIntervalMs) ? timeoutP : eMinKeepAliveIntervalMs;
 }
 
+void cSatipTuner::SetupTransport(int rtpPortP, int rtcpPortP, const char *streamAddrP, const char *sourceAddrP)
+{
+  cMutexLock MutexLock(&mutexM);
+  debug1("%s (%d, %d, %s, %s) [device %d]", __PRETTY_FUNCTION__, rtpPortP, rtcpPortP, streamAddrP, sourceAddrP, deviceIdM);
+  bool multicast = !isempty(streamAddrP);
+  // Adapt RTP to any transport media change
+  if (multicast != rtpM.IsMulticast() || rtpPortP != rtpM.Port()) {
+     cSatipPoller::GetInstance()->Unregister(rtpM);
+     rtpM.Close();
+     if (rtpPortP >= 0) {
+        if (multicast)
+           rtpM.OpenMulticast(rtpPortP, streamAddrP, sourceAddrP);
+        else
+           rtpM.Open(rtpPortP);
+        cSatipPoller::GetInstance()->Register(rtpM);
+        }
+     }
+  // Adapt RTCP to any transport media change
+  if (multicast != rtcpM.IsMulticast() || rtcpPortP != rtcpM.Port()) {
+     cSatipPoller::GetInstance()->Unregister(rtcpM);
+     rtcpM.Close();
+     if (rtcpPortP >= 0) {
+        if (multicast)
+           rtcpM.OpenMulticast(rtpPortP, streamAddrP, sourceAddrP);
+        else
+           rtcpM.Open(rtpPortP);
+        cSatipPoller::GetInstance()->Register(rtcpM);
+        }
+     }
+}
+
+cString cSatipTuner::GetBaseUrl(const char *addressP, const int portP)
+{
+  debug16("%s (%s, %d) [device %d]", __PRETTY_FUNCTION__, addressP, portP, deviceIdM);
+
+  if (portP != SATIP_DEFAULT_RTSP_PORT)
+     return cString::sprintf("rtsp://%s:%d/", addressP, portP);
+
+  return cString::sprintf("rtsp://%s/", addressP);
+}
+
 int cSatipTuner::GetId(void)
 {
   debug16("%s [device %d]", __PRETTY_FUNCTION__, deviceIdM);
@@ -375,6 +434,10 @@ bool cSatipTuner::SetSource(cSatipServer *serverP, const int transponderP, const
         // Update stream address and parameter
         streamAddrM = rtspM.RtspUnescapeString(*nextServerM.GetAddress());
         streamParamM = rtspM.RtspUnescapeString(parameterP);
+        streamPortM = nextServerM.GetPort();
+        // Modify parameter if required
+        if (nextServerM.IsQuirk(cSatipServer::eSatipQuirkForcePilot) && strstr(parameterP, "msys=dvbs2") && !strstr(parameterP, "plts="))
+           streamParamM = rtspM.RtspUnescapeString(*cString::sprintf("%s&plts=on", parameterP));
         // Reconnect
         RequestState(tsSet, smExternal);
         }
@@ -413,7 +476,7 @@ bool cSatipTuner::UpdatePids(bool forceP)
   cMutexLock MutexLock(&mutexM);
   if (((forceP && pidsM.Size()) || (pidUpdateCacheM.TimedOut() && (addPidsM.Size() || delPidsM.Size()))) &&
       !isempty(*streamAddrM) && (streamIdM > 0)) {
-     cString uri = cString::sprintf("rtsp://%s/stream=%d", *streamAddrM, streamIdM);
+     cString uri = cString::sprintf("%sstream=%d", *GetBaseUrl(*streamAddrM, streamPortM), streamIdM);
      bool useci = (SatipConfig.GetCIExtension() && currentServerM.HasCI());
      bool usedummy = currentServerM.IsQuirk(cSatipServer::eSatipQuirkPlayPids);
      if (forceP || usedummy) {
@@ -429,20 +492,30 @@ bool cSatipTuner::UpdatePids(bool forceP)
            uri = cString::sprintf("%s%sdelpids=%s", *uri, addPidsM.Size() ? "&" : "?", *delPidsM.ListPids());
         }
      if (useci) {
-        // CI extension parameters:
-        // - x_pmt : specifies the PMT of the service you want the CI to decode
-        // - x_ci  : specfies which CI slot (1..n) to use
-        //           value 0 releases the CI slot
-        //           CI slot released automatically if the stream is released,
-        //           but not when used retuning to another channel
-        int pid = deviceM->GetPmtPid();
-        if ((pid > 0) && (pid != pmtPidM)) {
-           int slot = deviceM->GetCISlot();
-           uri = cString::sprintf("%s&x_pmt=%d", *uri, pid);
-           if (slot > 0)
-              uri = cString::sprintf("%s&x_ci=%d", *uri, slot);
+        if (currentServerM.IsQuirk(cSatipServer::eSatipQuirkCiXpmt)) {
+           // CI extension parameters:
+           // - x_pmt : specifies the PMT of the service you want the CI to decode
+           // - x_ci  : specfies which CI slot (1..n) to use
+           //           value 0 releases the CI slot
+           //           CI slot released automatically if the stream is released,
+           //           but not when used retuning to another channel
+           int pid = deviceM->GetPmtPid();
+           if ((pid > 0) && (pid != pmtPidM)) {
+              int slot = deviceM->GetCISlot();
+              uri = cString::sprintf("%s&x_pmt=%d", *uri, pid);
+              if (slot > 0)
+                 uri = cString::sprintf("%s&x_ci=%d", *uri, slot);
+              }
+           pmtPidM = pid;
+           }
+        else if (currentServerM.IsQuirk(cSatipServer::eSatipQuirkCiTnr)) {
+           // CI extension parameters:
+           // - tnr : specifies a channel config entry
+           cString param = deviceM->GetTnrParameterString();
+           if (!isempty(*param) && strcmp(*tnrParamM, *param) != 0)
+              uri = cString::sprintf("%s&tnr=%s", *uri, *param);
+           tnrParamM = param;
            }
-        pmtPidM = pid;
         }
      pidUpdateCacheM.Set(ePidUpdateIntervalMs);
      if (!rtspM.Play(*uri))
@@ -463,7 +536,7 @@ bool cSatipTuner::KeepAlive(bool forceP)
      forceP = true;
      }
   if (forceP && !isempty(*streamAddrM)) {
-     cString uri = cString::sprintf("rtsp://%s/", *streamAddrM);
+     cString uri = GetBaseUrl(*streamAddrM, streamPortM);
      if (!rtspM.Options(*uri))
         return false;
      }
@@ -480,7 +553,7 @@ bool cSatipTuner::ReadReceptionStatus(bool forceP)
      forceP = true;
      }
   if (forceP && !isempty(*streamAddrM) && (streamIdM > 0)) {
-     cString uri = cString::sprintf("rtsp://%s/stream=%d", *streamAddrM, streamIdM);
+     cString uri = cString::sprintf("%sstream=%d", *GetBaseUrl(*streamAddrM, streamPortM), streamIdM);
      if (rtspM.Describe(*uri))
         return true;
      }
@@ -615,5 +688,5 @@ cString cSatipTuner::GetSignalStatus(void)
 cString cSatipTuner::GetInformation(void)
 {
   debug16("%s [device %d]", __PRETTY_FUNCTION__, deviceIdM);
-  return (currentStateM >= tsTuned) ? cString::sprintf("rtsp://%s/?%s [stream=%d]", *streamAddrM, *streamParamM, streamIdM) : "connection failed";
+  return (currentStateM >= tsTuned) ? cString::sprintf("%s?%s (%s) [stream=%d]", *GetBaseUrl(*streamAddrM, streamPortM), *streamParamM, *rtspM.GetActiveMode(), streamIdM) : "connection failed";
 }
